@@ -68,8 +68,8 @@ _build_time_vars = {'ABIFLAGS': '',
  'CONFINCLUDEDIR': '{BUILDROOT}/include',
  'CONFINCLUDEPY': '{BUILDROOT}/include/python3.10',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/home/runner/work/relenv/relenv/src/Python-3.10.17/coverage.info',
- 'COVERAGE_REPORT': '/home/runner/work/relenv/relenv/src/Python-3.10.17/lcov-report',
+ 'COVERAGE_INFO': '/home/runner/work/relenv/relenv/src/Python-3.10.19/coverage.info',
+ 'COVERAGE_REPORT': '/home/runner/work/relenv/relenv/src/Python-3.10.19/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-I. -I./Include -I{BUILDROOT}/include '
@@ -594,7 +594,7 @@ _build_time_vars = {'ABIFLAGS': '',
  'LLVM_PROF_FILE': '',
  'LLVM_PROF_MERGER': 'true',
  'LN': 'ln',
- 'LOCALMODLIBS': '-lreadline -ltinfow',
+ 'LOCALMODLIBS': '-lreadline -ltinfow  -lpanelw -lncursesw',
  'MACHDEP': 'linux',
  'MACHDEP_OBJS': '',
  'MACHDESTLIB': '{BUILDROOT}/lib/python3.10',
@@ -608,9 +608,10 @@ _build_time_vars = {'ABIFLAGS': '',
  'MODBUILT_NAMES': 'posix  errno  pwd  _sre  _codecs  _weakref  _functools  '
                    '_operator  _collections  _abc  itertools  atexit  _signal  '
                    '_stat  time  _thread  _locale  _io  faulthandler  '
-                   '_tracemalloc  _symtable  readline  xxsubtype',
+                   '_tracemalloc  _symtable  readline  _curses_panel  '
+                   'xxsubtype',
  'MODDISABLED_NAMES': '_tkinter  nsl  nis',
- 'MODLIBS': '-lreadline -ltinfow',
+ 'MODLIBS': '-lreadline -ltinfow  -lpanelw -lncursesw',
  'MODOBJS': 'Modules/posixmodule.o  Modules/errnomodule.o  '
             'Modules/pwdmodule.o  Modules/_sre.o  Modules/_codecsmodule.o  '
             'Modules/_weakref.o  Modules/_functoolsmodule.o  '
@@ -622,7 +623,7 @@ _build_time_vars = {'ABIFLAGS': '',
             'Modules/fileio.o Modules/bytesio.o Modules/bufferedio.o '
             'Modules/textio.o Modules/stringio.o  Modules/faulthandler.o  '
             'Modules/_tracemalloc.o  Modules/symtablemodule.o  '
-            'Modules/readline.o  Modules/xxsubtype.o',
+            'Modules/readline.o  Modules/_curses_panel.o  Modules/xxsubtype.o',
  'MODULE_OBJS': '\\',
  'MULTIARCH': 'x86_64-linux-gnu',
  'MULTIARCH_CPPFLAGS': '-DMULTIARCH=\\"x86_64-linux-gnu\\"',
@@ -800,7 +801,7 @@ _build_time_vars = {'ABIFLAGS': '',
  'USE_COMPUTED_GOTOS': 0,
  'VERSION': '3.10',
  'WHEEL_PKG_DIR': '',
- 'WINDOW_HAS_FLAGS': 1,
+ 'WINDOW_HAS_FLAGS': 0,
  'WITH_DECIMAL_CONTEXTVAR': 1,
  'WITH_DOC_STRINGS': 1,
  'WITH_DTRACE': 0,
@@ -812,14 +813,16 @@ _build_time_vars = {'ABIFLAGS': '',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/home/runner/work/relenv/relenv/src/Python-3.10.17',
- 'abs_srcdir': '/home/runner/work/relenv/relenv/src/Python-3.10.17',
+ 'abs_builddir': '/home/runner/work/relenv/relenv/src/Python-3.10.19',
+ 'abs_srcdir': '/home/runner/work/relenv/relenv/src/Python-3.10.19',
  'datarootdir': '{BUILDROOT}/share',
  'exec_prefix': '{BUILDROOT}',
  'prefix': '{BUILDROOT}',
  'srcdir': '.'}
 
-import pathlib, sys, platform, os
+import pathlib, sys, platform, os, logging
+
+log = logging.getLogger(__name__)
 
 def build_arch():
     machine = platform.machine()
@@ -853,7 +856,9 @@ else:
     DATA_DIR = DEFAULT_DATA_DIR
 
 buildroot = pydir.parent.parent
+
 toolchain = DATA_DIR / "toolchain" / get_triplet()
+
 build_time_vars = {}
 for key in _build_time_vars:
     val = _build_time_vars[key]
